@@ -8,12 +8,12 @@ from .field_walker import FieldWalker, FieldWriteError
 from .weighted import Weighted, _cmp_decimal
 from .queries import QueryFilter, ordering
 from ..types import (
-    Decimal128,
-    Timestamp,
+    bson_ as bson,
     string_types,
     is_numeric_type,
     is_duckument_type,
     is_integer_type,
+    keep,
 )
 
 
@@ -144,11 +144,6 @@ class Updator(object):
                 raise WriteError(msg, code=9)
 
             for field, value in cmd_doc.items():
-                if field == "_id":
-                    msg = ("Performing an update on the path '_id' would "
-                           "modify the immutable field '_id'")
-                    raise WriteError(msg, code=66)
-
                 for top in list(idnt_tops):
                     if "$[{}]".format(top) in field:
                         idnt_tops.remove(top)
@@ -177,6 +172,7 @@ class Updator(object):
         self.fields_to_update.append(field)
 
     def parse_set_on_insert(self, field, value, array_filters):
+        @keep(value)
         def _set_on_insert(fieldwalker):
             if self.__insert:
                 parse_set(field, value, array_filters)(fieldwalker)
@@ -192,6 +188,7 @@ def parse_inc(field, value, array_filters):
                "{{{0}: {1}}}".format(field, val_repr_))
         raise WriteError(msg, code=14)
 
+    @keep(value)
     def _inc(fieldwalker):
 
         def evaluator(node, inc_val):
@@ -205,15 +202,15 @@ def parse_inc(field, value, array_filters):
                 raise WriteError(msg, code=14)
 
             is_decimal128 = False
-            if isinstance(old_val, Decimal128):
+            if isinstance(old_val, bson.Decimal128):
                 is_decimal128 = True
                 old_val = old_val.to_decimal()
-            if isinstance(inc_val, Decimal128):
+            if isinstance(inc_val, bson.Decimal128):
                 is_decimal128 = True
                 inc_val = inc_val.to_decimal()
 
             if is_decimal128:
-                return Decimal128((old_val or 0) + inc_val)
+                return bson.Decimal128((old_val or 0) + inc_val)
             else:
                 return (old_val or 0) + inc_val
 
@@ -223,7 +220,7 @@ def parse_inc(field, value, array_filters):
 
 
 def parse_min(field, value, array_filters):
-
+    @keep(value)
     def _min(fieldwalker):
 
         def evaluator(node, min_val):
@@ -241,7 +238,7 @@ def parse_min(field, value, array_filters):
 
 
 def parse_max(field, value, array_filters):
-
+    @keep(value)
     def _max(fieldwalker):
 
         def evaluator(node, max_val):
@@ -266,6 +263,7 @@ def parse_mul(field, value, array_filters):
                "{{{0}: {1}}}".format(field, val_repr_))
         raise WriteError(msg, code=14)
 
+    @keep(value)
     def _mul(fieldwalker):
 
         def evaluator(node, mul_val):
@@ -279,15 +277,15 @@ def parse_mul(field, value, array_filters):
                 raise WriteError(msg, code=14)
 
             is_decimal128 = False
-            if isinstance(old_val, Decimal128):
+            if isinstance(old_val, bson.Decimal128):
                 is_decimal128 = True
                 old_val = old_val.to_decimal()
-            if isinstance(mul_val, Decimal128):
+            if isinstance(mul_val, bson.Decimal128):
                 is_decimal128 = True
                 mul_val = mul_val.to_decimal()
 
             if is_decimal128:
-                return Decimal128((old_val or 0) * mul_val)
+                return bson.Decimal128((old_val or 0) * mul_val)
             else:
                 return (old_val or 0.0) * mul_val
 
@@ -318,6 +316,7 @@ def parse_rename(field, new_field, array_filters):
                "same path: {0}: {1!r}".format(field, new_field))
         raise WriteError(msg, code=2)
 
+    @keep(new_field)
     def _rename(fieldwalker):
 
         probe = FieldWalker(fieldwalker.doc)
@@ -358,7 +357,7 @@ def parse_rename(field, new_field, array_filters):
 
 
 def parse_set(field, value, array_filters):
-
+    @keep(value)
     def _set(fieldwalker):
 
         _update(fieldwalker, field, value, None, array_filters)
@@ -367,7 +366,7 @@ def parse_set(field, value, array_filters):
 
 
 def parse_unset(field, _, array_filters):
-
+    @keep(field)
     def _unset(fieldwalker):
 
         _drop(fieldwalker, field, array_filters)
@@ -378,7 +377,7 @@ def parse_unset(field, _, array_filters):
 def parse_currentDate(field, value, array_filters):
     date_type = {
         "date": datetime.utcnow(),
-        "timestamp": Timestamp(datetime.utcnow(), 1),
+        "timestamp": bson.Timestamp(datetime.utcnow(), 1),
     }
 
     if not isinstance(value, bool):
@@ -402,6 +401,7 @@ def parse_currentDate(field, value, array_filters):
     else:
         value = date_type["date"]
 
+    @keep(value)
     def _currentDate(fieldwalker):
         parse_set(field, value, array_filters)(fieldwalker)
 
@@ -417,6 +417,7 @@ def parse_add_to_set(field, value_or_each, array_filters):
         value = value_or_each
         run_each = False
 
+    @keep(value)
     def _add_to_set(fieldwalker):
 
         def evaluator(node, new_elem):
@@ -458,6 +459,7 @@ def parse_pop(field, value, array_filters):
         if value not in (1.0, -1.0):
             raise WriteError(msg_raw.format(field, value), code=9)
 
+    @keep(value)
     def _pop(fieldwalker):
 
         def evaluator(node, pop_ind):
@@ -494,6 +496,7 @@ def parse_pull(field, value_or_conditions, array_filters):
     else:
         queryfilter = QueryFilter({field: value_or_conditions})
 
+    @keep(queryfilter)
     def _pull(fieldwalker):
 
         def evaluator(node, _):
@@ -527,6 +530,7 @@ def parse_push(field, value_or_each, array_filters):
         value = value_or_each
         run_each = False
 
+    @keep(value)
     def _push(fieldwalker):
 
         def evaluator(node, new_elem):
@@ -560,6 +564,7 @@ def parse_pull_all(field, value, array_filters):
                "".format(value_type))
         raise WriteError(msg, code=2)
 
+    @keep(value)
     def _pull_all(fieldwalker):
 
         def evaluator(node, pull_list):
@@ -574,7 +579,7 @@ def parse_pull_all(field, value, array_filters):
 
             def convert(lst):
                 for val in lst:
-                    if isinstance(val, Decimal128):
+                    if isinstance(val, bson.Decimal128):
                         yield _cmp_decimal(val)
                     else:
                         yield val
