@@ -6,10 +6,7 @@ import contextlib
 from collections import OrderedDict
 
 from ..base import WriteConcern
-from ..types import (
-    unicode_,
-    document_encode,
-)
+from ..types import unicode_, bson_ as bson
 from . import (
     AbstractStorage,
     AbstractDatabase,
@@ -339,16 +336,18 @@ class SQLiteCollection(AbstractCollection):
     def write_one(self, doc, check_keys=True):
         """
         """
+        _id = doc["_id"]
         try:
             self._conn.write_one(
                 self._col_path,
-                (str(doc["_id"]), self._encode_doc(doc, check_keys),),
+                (bson.id_encode(_id),
+                 self._encode_doc(doc, check_keys),),
                 self.wconcern
             )
         except sqlite3.IntegrityError:
             raise StorageDuplicateKeyError()
 
-        return doc["_id"]
+        return _id
 
     @_ensure_table
     def write_many(self, docs, check_keys=True, ordered=True):
@@ -359,13 +358,14 @@ class SQLiteCollection(AbstractCollection):
         keys = self._conn.read_all_keys(self._col_path)
         has_duplicated_key = False
         for doc in docs:
-            id = doc["_id"]
-            if str(id) in keys or str(id) in _docs:
+            _id = doc["_id"]
+            b_id = bson.id_encode(_id)
+            if b_id in keys or b_id in _docs:
                 has_duplicated_key = True
                 break
 
-            _docs[str(id)] = self._encode_doc(doc, check_keys)
-            ids.append(id)
+            _docs[b_id] = self._encode_doc(doc, check_keys)
+            ids.append(_id)
 
         self._conn.write_many(
             self._col_path,
@@ -383,7 +383,7 @@ class SQLiteCollection(AbstractCollection):
         """
         self._conn.update_one(
             self._col_path,
-            (self._encode_doc(doc), str(doc["_id"])),
+            (self._encode_doc(doc), bson.id_encode(doc["_id"])),
             self.wconcern
         )
 
@@ -392,21 +392,22 @@ class SQLiteCollection(AbstractCollection):
         """
         self._conn.update_many(
             self._col_path,
-            [(self._encode_doc(doc), str(doc["_id"])) for doc in docs],
+            [(self._encode_doc(doc), bson.id_encode(doc["_id"]))
+             for doc in docs],
             self.wconcern
         )
 
     def delete_one(self, id):
         self._conn.delete_one(
             self._col_path,
-            (str(id),),
+            (bson.id_encode(id),),
             self.wconcern
         )
 
     def delete_many(self, ids):
         self._conn.delete_many(
             self._col_path,
-            [(str(id),) for id in ids],
+            [(bson.id_encode(id),) for id in ids],
             self.wconcern
         )
 
